// $Id: MSXBunsetsu.cc,v 1.1 2002/12/17 19:27:14 m9710797 Exp $

#include "MSXBunsetsu.hh"
#include "CPU.hh"


MSXBunsetsu::MSXBunsetsu(MSXConfig::Device *config, const EmuTime &time)
	: MSXDevice(config, time), MSXMemDevice(config, time),
	  rom(config, time),
	  jisyoRom(config->getParameter("jisyofilename"), time)
{
	reset(time);
}

MSXBunsetsu::~MSXBunsetsu()
{
}

void MSXBunsetsu::reset(const EmuTime &time)
{
	jisyoAddress = 0;
}

byte MSXBunsetsu::readMem(word address, const EmuTime &time)
{
	byte result;
	if (address == 0xBFFF) {
		result = jisyoRom.read(jisyoAddress);
		jisyoAddress = (jisyoAddress + 1) & 0x1FFFF;
	} else if ((0x4000 <= address) && (address < 0xC000)) {
		result = rom.read(address - 0x4000);
	} else {
		result = 0xFF;
	}
	return result;
}

void MSXBunsetsu::writeMem(word address, byte value, const EmuTime &time)
{
	switch (address) {
		case 0xBFFC:
			jisyoAddress = (jisyoAddress & 0x1FF00) | value;
			break;
		case 0xBFFD:
			jisyoAddress = (jisyoAddress & 0x100FF) | (value << 8);
			break;
		case 0xBFFE:
			jisyoAddress = (jisyoAddress & 0x1FFFF) | 
			               ((value & 1) << 16);
			break;
	}
}

byte* MSXBunsetsu::getReadCacheLine(word start)
{
	if ((start & 0xBFFC & CPU::CACHE_LINE_HIGH) == 
	    (0xBFFC & CPU::CACHE_LINE_HIGH)) {
		return NULL;
	} else {
		return rom.getBlock(start - 0x4000);
	}
}
